
//A smart bitmap is a bitmap with a corresponding converter.

#define smartBitmapOK 0
#define smartBitmapFileNotSmart 1
#define smartBitmapFileOpenFailure 2
#define smartBitmapReadFailure 3
#define smartBitmapWriteFailure 4
#define smartBitmapSpaceFailure 5
#define smartBitmapOperationFailure 6

char *errorMessages [] = {
	"OK",
	"Not a smart bitmap",
	"Could not open for reading",
	"Read error",
	"Write error",
	"out of space",
	"readOpen/write attempt or writeOpen/read attempt"};

#define smartBitmapCanOpen 0
#define smartBitmapCanRead 1
#define smartBitmapCanWrite 2
#define smartBitmapClosed 3

struct Pixel {
	unsigned char red;
	unsigned char green;
	unsigned char blue;
	unsigned char alpha;
};

typedef void (ReadOpenFunction) (class SmartBitmap &bitmap, long &width, long &height);
typedef void (WriteOpenFunction) (class SmartBitmap &bitmap, long width, long height);
typedef void (CloseFunction) (class SmartBitmap &bitmap);
typedef void (PixelGetFunction) (class SmartBitmap &bitmap, long x, long y, Pixel &pixel);
typedef void (PixelGetAllFunction) (class SmartBitmap &bitmap, Pixel pixel []);
typedef void (PixelSetFunction) (class SmartBitmap &bitmap, long x, long y, Pixel &pixel);
typedef void (PixelSetAllFunction) (class SmartBitmap &bitmap, Pixel pixel []);

class SmartBitmap {
public:
	long width; long height;
	char fileName [256];
	long errorCode;
	char errorMessage [256];
	long status;
	void *privateData;
	HINSTANCE dll;
	ReadOpenFunction *readOpenFunction;
	WriteOpenFunction *writeOpenFunction;
	CloseFunction *closeFunction;
	PixelGetFunction *pixelGetFunction;
	PixelGetAllFunction *pixelGetAllFunction;
	PixelSetFunction *pixelSetFunction;
	PixelSetAllFunction *pixelSetAllFunction;

	SmartBitmap (char *fileName);
	~SmartBitmap ();
};

